==> bubble_aide-0.1.12-py3-none-any.whl: bubble_aide/delegate.py <==
from typing import TYPE_CHECKING
from bubble.datastructures import AttributeDict

from bubble_aide.abc.module import PrecompileContract
from bubble_aide.utils.wrapper import contract_transaction

if TYPE_CHECKING:
    from bubble_aide import Aide


class Delegate(PrecompileContract):

    def __init__(self, aide: "Aide"):
        super().__init__(aide)
        self.address = self.aide.web3.dpos.delegate.ADDRESS

    @property
    def staking_block_number(self):
        return self.aide.staking.staking_info.StakingBlockNum

    @contract_transaction()
    def delegate(self,
                 amount=None,
                 balance_type=0,
                 node_id=None,
                 txn=None,
                 private_key=None,
                 ):
        """ 委托节点，以获取节点的奖励分红
        """
        amount = amount or self.aide.economic.delegate_limit
        node_id = node_id or self.aide.constant.node_id
        return self.aide.web3.dpos.delegate.delegate(node_id, balance_type, amount)

    @contract_transaction(1005)
    def withdrew_delegate(self,
                          amount=0,
                          staking_block_identifier=None,
                          node_id=None,
                          txn=None,
                          private_key=None,
                          ):
        """
        撤回对节点的委托，可以撤回部分委托
        注意：因为节点可能进行过多次质押/撤销质押，会使得委托信息遗留，因此撤回委托时必须指定节点质押区块
        """
        node_id = node_id or self.aide.constant.node_id
        amount = amount or self.aide.economic.delegate_limit
        staking_block_identifier = staking_block_identifier or self.aide.staking.staking_info.StakingBlockNum

        return self.aide.web3.dpos.delegate.withdrew_delegate(node_id,
                                                              staking_block_identifier,
                                                              amount,
                                                              )

    @contract_transaction(1006)
    def redeem_delegate(self,
                        txn=None,
                        private_key=None,
                        ):
        """
        领取已经解锁的委托金
        """
        return self.aide.web3.dpos.delegate.redeem_delegate()

    def get_delegate_info(self,
                          address=None,
                          node_id=None,
                          staking_block_identifier=None,
                          ):
        """ 获取地址对某个节点的某次质押的委托信息
        注意：因为节点可能进行过多次质押/撤销质押，会使得委托信息遗留，因此获取委托信息时必须指定节点质押区块
        """
        if self.aide.account:
            address = address or self.aide.account.address
        node_id = node_id or self.aide.constant.node_id
        staking_block_identifier = staking_block_identifier or self.aide.staking.staking_info.StakingBlockNum

        delegate_info = self.aide.web3.dpos.delegate.get_delegate_info(address, node_id, staking_block_identifier).call()
        if delegate_info == 'Query delegate info failed:Delegate info is not found':
            return None
        else:
            return DelegateInfo(delegate_info)

    def get_delegate_lock_info(self,
                               address=None,
                               ):
        """ 获取地址处于锁定期的委托信息
        """
        address = address or self.aide.account.address

        delegate_lock_info = self.aide.web3.dpos.delegate.get_delegate_lock_info(address).call()
        # todo: 根据实际情况补全
        if delegate_lock_info == 'Query delegate info failed:Delegate info is not found':
            return None
        else:
            return delegate_lock_info

    def get_delegate_list(self, address=None):
        """ 获取地址的全部委托信息
        """
        if self.aide.account:
            address = address or self.aide.account.address
        delegate_list = self.aide.web3.dpos.delegate.get_delegate_list(address).call()
        if delegate_list == 'Retreiving delegation related mapping failed:RelatedList info is not found':
            return []
        else:
            return [DelegateInfo(delegate_info) for delegate_info in delegate_list]


class DelegateInfo(AttributeDict):
    """ 委托信息的属性字典类
    """
    Addr: str
    NodeId: str
    StakingBlockNum: int
    DelegateEpoch: int
    Released: int
    ReleasedHes: int
    RestrictingPlan: int
    RestrictingPlanHes: int
    CumulativeIncome: int
    LockReleasedHes: int
    LockRestrictingPlanHes: int

==> bubble_aide-2.1.6-py3-none-any.whl: bubble_aide/delegate.py <==
from typing import TYPE_CHECKING
from bubble.datastructures import AttributeDict

from bubble_aide.abc.module import PrecompileContract
from bubble_aide.utils.wrapper import contract_transaction

if TYPE_CHECKING:
    from bubble_aide import Aide


class Delegate(PrecompileContract):

    def __init__(self, aide: "Aide"):
        super().__init__(aide)
        self.address = self.aide.web3.dpos.delegate.ADDRESS

    @property
    def staking_block_number(self):
        return self.aide.staking.staking_info.StakingBlockNum

    @contract_transaction()
    def delegate(self,
                 amount=None,
                 balance_type=0,
                 node_id=None,
                 txn=None,
                 private_key=None,
                 ):
        """ Delegate nodes to obtain their reward dividends
        """
        amount = amount or self.aide.economic.delegate_limit
        node_id = node_id or self.aide.constant.node_id
        return self.aide.web3.dpos.delegate.delegate(node_id, balance_type, amount)

    @contract_transaction(1005)
    def withdrew_delegate(self,
                          amount=0,
                          staking_block_identifier=None,
                          node_id=None,
                          txn=None,
                          private_key=None,
                          ):
        """
        Revoke delegation to node, partial delegation can be revoked
        Note：Because the node may have pledged/revoked multiple times, which may leave the entrusted information behind, it is necessary to specify the node's pledge block when revoking the entrustment
        """
        node_id = node_id or self.aide.constant.node_id
        amount = amount or self.aide.economic.delegate_limit
        staking_block_identifier = staking_block_identifier or self.aide.staking.staking_info.StakingBlockNum

        return self.aide.web3.dpos.delegate.withdrew_delegate(node_id,
                                                              staking_block_identifier,
                                                              amount,
                                                              )

    @contract_transaction(1006)
    def redeem_delegate(self,
                        txn=None,
                        private_key=None,
                        ):
        """
        Claim unlocked commission money
        """
        return self.aide.web3.dpos.delegate.redeem_delegate()

    def get_delegate_info(self,
                          address=None,
                          node_id=None,
                          staking_block_identifier=None,
                          ):
        """ Obtain the delegation information for a certain pledge of an address to a node
        Note: Because the node may have pledged/revoked multiple times, it may leave the entrusted information behind. Therefore, when obtaining the entrusted information, it is necessary to specify the node's pledge block
        """
        if self.aide.account:
            address = address or self.aide.account.address
        node_id = node_id or self.aide.constant.node_id
        staking_block_identifier = staking_block_identifier or self.aide.staking.staking_info.StakingBlockNum

        delegate_info = self.aide.web3.dpos.delegate.get_delegate_info(address, node_id,
                                                                       staking_block_identifier).call()
        if delegate_info == 'Query delegate info failed:Delegate info is not found':
            return None
        else:
            return DelegateInfo(delegate_info)

    def get_delegate_lock_info(self,
                               address=None,
                               ):
        """ Obtain delegation information for addresses that are in the lock period
        """
        address = address or self.aide.account.address

        delegate_lock_info = self.aide.web3.dpos.delegate.get_delegate_lock_info(address).call()
        # todo: Complete according to actual situation
        if delegate_lock_info == 'Query delegate info failed:Delegate info is not found':
            return None
        else:
            return delegate_lock_info

    def get_delegate_list(self, address=None):
        """ Obtain all delegation information for the address
        """
        if self.aide.account:
            address = address or self.aide.account.address
        delegate_list = self.aide.web3.dpos.delegate.get_delegate_list(address).call()
        if delegate_list == 'Retreiving delegation related mapping failed:RelatedList info is not found':
            return []
        else:
            return [DelegateInfo(delegate_info) for delegate_info in delegate_list]


class DelegateInfo(AttributeDict):
    """ Attribute dictionary class for delegation information
    """
    Addr: str
    NodeId: str
    StakingBlockNum: int
    DelegateEpoch: int
    Released: int
    ReleasedHes: int
    RestrictingPlan: int
    RestrictingPlanHes: int
    CumulativeIncome: int
    LockReleasedHes: int
    LockRestrictingPlanHes: int
